from __future__ import unicode_literals
import json

from django.template import Context
from django.test import TestCase

from django_jsx.templatetags.jsx import serialize_opportunistically


class SerializeOpportunisticallyTest(TestCase):
    def test_no_expressions(self):
        obj = {
            'a': 1,
            'b': 2,
            'c': 3
        }
        result = serialize_opportunistically(obj, [])
        self.assertEqual({}, json.loads(result))

    def test_simple_expressions(self):
        obj = {
            'a': 1,
            'b': 2,
            'c': 3
        }
        result = serialize_opportunistically(obj, ['a', 'b'])
        expect = {
            'a': 1,
            'b': 2,
        }
        self.assertEqual(expect, json.loads(result))

    def test_missing_variable(self):
        """
        In Django templates, if a variable is not defined in the context, then
        Django sets that variable to the special value ``string_if_invalid``
        which is normally the empty string, but is customizable in Django
        settings TEMPLATES['OPTIONS']['string_if_invalid']. Django doesn't
        raise a user-visible error, so we should do the same thing in
        django_jsx.
        """
        result = serialize_opportunistically(Context(), ['no.such.variable'])
        expect = {
            'no': {
                'such': {
                    'variable': ''
                }
            }
        }
        self.assertEqual(expect, json.loads(result))

    def test_nested_expressions(self):
        obj = {
            'a': {
                'd': 4,
                'f': {
                    'g': 'Hello'
                }
            },
            'b': 2,
            'c': {
                'e': 5
            }
        }
        result = serialize_opportunistically(Context(obj), ['a.d', 'b', 'c.e', 'a.f', 'a.f.g'])
        expect = {
            'a': {

                'd': 4,
                'f': {
                    'g': 'Hello'
                }
            },
            'b': 2,
            'c': {
                'e': 5
            },
        }
        self.assertEqual(expect, json.loads(result))

    def test_top_level_object_doesnt_clobber_nested_expressions(self):
        # create an object with some attributes
        class Location(dict):
            full_name = 'New York'
        location = Location()

        # the order of expressions is important here
        expressions = [
            'location.full_name',  # set up the nested dict
            'location',            # this should not clobber the nested dict
        ]
        obj = {'location': location}
        result = serialize_opportunistically(Context(obj), expressions)
        expect = {
            'location': {
                'full_name': 'New York',
            }
        }
        self.assertEqual(expect, json.loads(result))

    def test_with_callable(self):
        def call1():
            return "called 1"

        def call2():
            return "called 2"

        obj = {
            'call1': call1,
            'a': {
                'b': call2,
                'c': call1,
            }
        }
        result = serialize_opportunistically(obj, ['call1', 'a.b'])
        expect = {
            'call1': 'called 1',
            'a': {
                'b': 'called 2'
            }
        }
        self.assertEqual(expect, json.loads(result))



"""Tests for finding all the template files"""
from __future__ import unicode_literals
import os.path

from django.test import TestCase
from django.test import override_settings

from django_jsx.management.commands.compilejsx import list_template_files


class TemplateFindingTest(TestCase):
    @override_settings(
        INSTALLED_APPS=['tests'],
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'APP_DIRS': True,
            },
        ])
    def test_finding_templates_with_test_app(self):
        # If 'tests' is the only installed app, we find its templates
        this_dir = os.path.dirname(__file__)
        result = list_template_files()
        expected = [
            os.path.join(this_dir, "templates", filename)
            for filename in
            [
                "test_file_B.html",
                "test_file_A.txt",
                "test_dir_C/test_file_D.zip",
            ]
        ]
        self.assertEqual(set(expected), set(result))

    @override_settings(
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'APP_DIRS': True,
            },
        ],
        INSTALLED_APPS=[])
    def test_finding_templates_without_test_app(self):
        # If there are no installed apps, we don't find any templates
        result = list_template_files()
        expected = []
        self.assertEqual(set(expected), set(result))

    @override_settings(
        INSTALLED_APPS=['tests'],
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [os.path.join(os.path.dirname(__file__), "more_templates")],
                'APP_DIRS': True,
            },
        ])
    def test_finding_templates_in_additional_dirs(self):
        # If the user configured additional dirs to find templates in, we spot them
        result = list_template_files()
        this_dir = os.path.dirname(__file__)
        self.assertEqual(
            set(result),
            {
                os.path.join(this_dir, "templates", filename)
                for filename in
                [
                    "test_file_B.html",
                    "test_file_A.txt",
                    "test_dir_C/test_file_D.zip",
                ]
            } | {
                os.path.join(this_dir, "more_templates", "another_template.html")
            }
        )

    @override_settings(
        INSTALLED_APPS=['test'],
        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [os.path.join(os.path.dirname(__file__), "more_templates")],
                'APP_DIRS': False,
            },
        ])
    def test_finding_templates_with_app_dirs_false(self):
        this_dir = os.path.dirname(__file__)
        result = list_template_files()
        expected = [os.path.join(this_dir, "more_templates", "another_template.html")]
        self.assertEqual(set(expected), set(result))



from __future__ import unicode_literals

import hashlib
import io
import os
import sys
import tempfile

from django.core.management import call_command
from django.test import TestCase

from django_jsx.management.commands.compilejsx import compile_templates, END_JS, START_JS


class CompileJSXTest(TestCase):
    """
    Tests for the compilejsx management command, which looks at all the
    template files and emits a jsx_registry.jsx file with information about
    the JSX blocks in the templates, and some JavaScript code to make use of the
    information.
    """
    @classmethod
    def setUpClass(cls):
        cls.files_to_delete = []

    @classmethod
    def tearDownClass(cls):
        for fn in cls.files_to_delete:
            try:
                os.remove(fn)
            except Exception as e:
                print(e)
        delattr(cls, 'files_to_delete')

    @classmethod
    def make_testfile(cls):
        """Returns name of the test file"""
        (filehandle, filename) = tempfile.mkstemp()
        os.close(filehandle)
        cls.files_to_delete.append(filename)
        return filename

    def test_invoking_for_stdout(self):
        # Default output is to stdout
        out = io.StringIO()
        orig_out = sys.stdout
        try:
            sys.stdout = out
            call_command('compilejsx')
            self.assertIn(START_JS, out.getvalue())
        finally:
            sys.stdout = orig_out

    def test_invoking_to_output_file(self):
        # --output sends output to named file
        filename = type(self).make_testfile()
        call_command('compilejsx', output=filename)
        output = open(filename, "rb").read().decode('utf-8')
        self.assertIn(START_JS, output)

    def try_it(self, test_content, expected_result, raw=False):
        # Make template file containing a jsx block whose body is `test_content`, run
        # compilejsx, and make sure the output is `expected_result`.  Or if `raw` is true,
        # then `test_content` is the entire content of the test file to compile.
        filename = type(self).make_testfile()
        expected_result = expected_result.replace('{filename}', filename)
        if raw:
            test_text = test_content
        else:
            test_text = "{% jsx %}" + test_content + "{% endjsx %}"
        with open(filename, "w") as f:
            f.write(test_text)

        # "Compile" it
        output = io.StringIO()
        compile_templates([filename], output)

        # Strip boilerplate to simplify checking
        start = len(START_JS) + 1
        end = 0 - (len(END_JS) + 1)
        result = output.getvalue()[start:end - 1]

        self.maxDiff = None
        self.assertEqual(expected_result, result)

    def test_empty_template(self):
        # If template is empty, output is just the boilerplate.
        # Make empty file
        filename = type(self).make_testfile()
        with open(filename, "w"):
            pass
        # "Compile" it
        output = io.StringIO()
        compile_templates([filename], output)

        # Check boilerplate
        self.assertTrue(output.getvalue().startswith(START_JS + "\n"))
        self.assertTrue(output.getvalue().endswith(END_JS + "\n"))

        # Strip boilerplate to simplify checking what's not boilerplate
        start = len(START_JS) + 1
        end = 0 - (len(END_JS) + 1)
        result = output.getvalue()[start:end - 1]
        self.assertEqual('', result)

    def test_template_with_empty_jsx_block(self):
        # If the block is empty, the output is pretty minimal

        test_content = ''
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
return ();
}''' % sha1
        self.try_it(test_content, expected)

    def test_template_with_minimal_component(self):
        # If the block just has a minimal React component, the output includes
        # a jsx_registry entry for it.
        test_content = '<NeatThing/>'
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'NeatThing')) var {NeatThing} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)
        self.try_it(test_content, expected)

    def test_template_with_component_with_static_property(self):
        # Static properties don't change the output
        test_content = '<NiftyFeature foo="bar"/>'
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'NiftyFeature')) var {NiftyFeature} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)
        self.try_it(test_content, expected)

    def test_template_with_component_with_variable_property(self):
        # Variable properties don't change the output
        test_content = '<WonderBar foo="{{ ctx.bar }}"/>'
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'WonderBar')) var {WonderBar} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)
        self.try_it(test_content, expected)

    def test_template_with_component_with_expression_property(self):
        # Expressions in properties don't change the output
        test_content = '<Component foo="{{ ctx.bar ? 3 : ctx.zip }}"/>'
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'Component')) var {Component} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)
        self.try_it(test_content, expected)

    def test_template_with_component_with_deep_variable(self):
        # Variable properties don't change the output
        test_content = '<Component foo="{{ ctx.foo.bar.baz }}"/>'
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'Component')) var {Component} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)
        self.try_it(test_content, expected)

    def test_template_with_nested_html(self):
        # Each tag level contributes to the output. compilejsx doesn't know or care
        # which tags are React components.
        test_content = '''<div id="search-mnt" className="search-section-mnt">
        <MobileModalSectionSearch
            section="attraction"
            sectionLabel="Attractions"
            currentLocation={ctx.location ? ctx.location.full_name : null}
            mapLink={ctx.map_link}
            useDistance={ctx.location && ctx.location.kind === 'city'}
            subCat={ctx.type_slug || ""}
        />
    </div>'''
        sha1 = hashlib.sha1(test_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'MobileModalSectionSearch')) var {MobileModalSectionSearch} = COMPONENTS;
if (Object.hasOwnProperty.call(COMPONENTS, 'div')) var {div} = COMPONENTS;
return (%s);
}''' % (sha1, test_content)  # noqa (long line hard to avoid here)
        self.try_it(test_content, expected)

    def test_duplicate_blocks_with_different_contexts(self):
        # compilejsx comes up with the same jsx_registry entry repeatedly if there are multiple
        # blocks with the same content but with different contexts.  But this is okay, as
        # the rendered template will have a tag for each occurrence of the block, each
        # with its own unique context, and the JavaScript will render a component for
        # each one using that context.
        block_content = '<Component prop={ctx.foo}/>'
        test_content = '''{% load jsx %}
        {% with foo=1 %}
            {% jsx %}BLOCK_CONTENT{% endjsx %}
        {% endwith %}
        {% with foo=2 %}
            {% jsx %}BLOCK_CONTENT{% endjsx %}
        {% endwith %}
        '''.replace("BLOCK_CONTENT", block_content)
        sha1 = hashlib.sha1(block_content.encode('utf-8')).hexdigest()
        expected = '''/* {filename} */
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'Component')) var {Component} = COMPONENTS;
return (%s);
}
jsx_registry["%s"] = (COMPONENTS, ctx) => {
if (Object.hasOwnProperty.call(COMPONENTS, 'Component')) var {Component} = COMPONENTS;
return (%s);
}''' % (sha1, block_content, sha1, block_content)
        self.try_it(test_content, expected, raw=True)



# SETTINGS file for running tests on this pluggable application

# https://docs.djangoproject.com/en/1.10/topics/testing/advanced/#using-the-django-test-runner-to-test-reusable-applications

SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
    "tests",
    "django_jsx",
]

# We need a database configuration or Django complains, but we're not
# actually using the database. An in-memory Sqlite database should
# be very light-weight and not leave anything behind after testing.
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ':memory:',
    }
}


# COPIED from project template in Django 1.10
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]



from __future__ import unicode_literals

import hashlib
import json
import re
from django.template import Context, Engine
from django.template import TemplateSyntaxError
from django.test import TestCase

from django_jsx.templatetags.jsx import set_nested


RESULT_REGEX = re.compile(
    r'<script type="script/django-jsx" data-sha1="(?P<sha1>[0-9a-f]+)" '
    r'data-ctx="(?P<ctx>.*)"></script>')

DEFAULT_CONTEXT = {
    'False': False,
    'True': True,
    'None': None
}

ENGINE = Engine.get_default()


def unescape(s):
    return s.replace('&amp;', '&').replace('&lt;', '<')\
        .replace('&gt;', '>').replace('&quot;', '"').replace('&#39;', "'")


class SetNestedTest(TestCase):
    def test_simple_key(self):
        d = {}
        set_nested(d, 'foo', 3)
        self.assertEqual({'foo': 3}, d)

    def test_one_level(self):
        d = {}
        set_nested(d, 'foo.bar', 3)
        self.assertEqual({'foo': {'bar': 3}}, d)

    def test_two_levels(self):
        d = {}
        set_nested(d, 'foo.bar.baz', 3)
        self.assertEqual({'foo': {'bar': {'baz': 3}}}, d)

    def test_with_existing_stuff(self):
        d = {'one': 1, 'foo': {'baz': 2}}
        set_nested(d, 'foo.bar', 3)
        self.assertEqual({'one': 1, 'foo': {'bar': 3, 'baz': 2}}, d)

    def test_with_existing_object(self):
        """
        If a top level item to be serialized is an object, we shouldn't fail at
        trying to set the lower level item.
        """
        d = {'foo': object()}
        set_nested(d, 'foo.bar', 3)
        self.assertEqual({'foo': {'bar': 3}}, d)

    def test_top_level_item_doesnt_clobber_nested(self):
        # foo.bar has previously been set
        d = {'foo': {'bar': 3}}
        # if we later try to set foo, we shouldn't clobber foo.bar
        set_nested(d, 'foo', object())
        self.assertEqual({'foo': {'bar': 3}}, d)


class JsxTagTest(TestCase):
    def test_loading_tags(self):
        # We can `load` the tag library
        engine = Engine.get_default()
        template_object = engine.from_string("{% load jsx %}")
        result = template_object.render(Context({}))
        self.assertEqual("", result)

    def try_it(self, content, expected_ctx, raw=False, context=None):
        # Assert that if we do a {% jsx %} block with the given content, that we
        # get the standard empty script tag in the output, the data-sha1 attribute
        # has the sha1 digest of the content, and the `data-ctx` is
        # a JSON-encoding of `expected_ctx`.
        # If `raw` is True, use `content` as the entire template content, not just
        # the insides of a jsx block, and just return the result rather than trying
        # to validate it.
        # If `context` is given, use it as the template context.
        if raw:
            template_content = content
        else:
            template_content = "{% load jsx %}{% jsx %}" + content + "{% endjsx %}"
        template_object = ENGINE.from_string(template_content)
        result = template_object.render(Context(context or {}))
        if raw:
            return result
        else:
            m = RESULT_REGEX.match(result)
            self.assertEqual(m.group('sha1'), hashlib.sha1(content.encode('utf-8')).hexdigest())
            ctx = json.loads(unescape(m.group('ctx')))
            self.assertEqual(ctx, expected_ctx)

    def test_empty_tag(self):
        # No content -> no context
        expected_ctx = {}
        self.try_it('', expected_ctx)

    def test_empty_component(self):
        # No references to context -> no context
        content = '<Component/>'
        expected_ctx = {}
        self.try_it(content, expected_ctx)

    def test_component_with_properties_using_context(self):
        # Refer to context -> context we used gets included
        content = '''<Component prop1="foo" prop2={ctx.False} prop3={not ctx.True}/>'''
        expected_ctx = {'False': False, 'True': True}
        self.try_it(content, expected_ctx)

    def test_referring_to_elements_numerically(self):
        content = '''<Component prop2={ctx.list.0}/>'''
        context = {'list': [1, 2, 3]}
        expected_ctx = {'list': {'0': 1}}
        self.try_it(content, expected_ctx, context=context)

    def test_missing_variables(self):
        "If variable is missing, set it to empty string (by default)."
        content = '''<Component prop2={ctx.does.not.exist}/>'''
        context = {}
        expected_ctx = {'does': {'not': {'exist': ''}}}
        self.try_it(content, expected_ctx, context=context)

    def test_missing_variables_with_string_if_invalid_set(self):
        "If variable is missing, use Engine's string_if_invalid value."
        ENGINE.string_if_invalid = 'hey, missing var -> %s'
        content = '''<Component prop2={ctx.does.not.exist}/>'''
        context = {}
        expected_ctx = {'does': {'not': {'exist': 'hey, missing var -> does.not.exist'}}}
        self.try_it(content, expected_ctx, context=context)

    # SHOULD NOT BE ALLOWED - compilejsx will reject
    # def test_two_identical_blocks_with_different_contexts(self):
    #     # If an identical block is repeated with different context values, we
    #     # can't handle it, so it should report an error and force the user to make
    #     # the blocks different.
    #     block_content = '<Component prop={ctx.foo}/>'
    #     test_content = '''{% spaceless %}
    #     {% load jsx %}
    #     {% with foo=1 %}
    #         {% jsx %}BLOCK_CONTENT{% endjsx %}
    #     {% endwith %}
    #     {% with foo=2 %}
    #         {% jsx %}BLOCK_CONTENT{% endjsx %}
    #     {% endwith %}
    #     {% endspaceless %}'''.replace("BLOCK_CONTENT", block_content)
    #     result = self.try_it(test_content, None, raw=True)
    #     sha1 = hashlib.sha1(block_content.encode('utf-8')).hexdigest()
    #     expected_output = (
    #         '<script type="script/django-jsx" data-sha1="SHA1" data-ctx="{"foo": 1}"></script>'
    #         '<script type="script/django-jsx" data-sha1="SHA1" data-ctx="{"foo": 2}"></script>'
    #         .replace('SHA1', sha1))
    #     self.assertEqual(expected_output, unescape(result))

    def test_block_in_loop(self):
        # We can put a block in a loop, and we get an output tag for each loop iteration
        test_content = '''{% spaceless %}
        {% load jsx %}
        {% for i in values %}
            {% jsx %}<Component key={ ctx.i }/>{% endjsx %}
        {% endfor %}
        {% endspaceless %}'''
        result = self.try_it(test_content, None, raw=True, context={'values': [1, 2, 3]})
        sha1 = hashlib.sha1('<Component key={ ctx.i }/>'.encode('utf-8')).hexdigest()
        expected_output = (
            '<script type="script/django-jsx" data-sha1="SHA1" data-ctx="{"i": 1}"></script>'
            '<script type="script/django-jsx" data-sha1="SHA1" data-ctx="{"i": 2}"></script>'
            '<script type="script/django-jsx" data-sha1="SHA1" data-ctx="{"i": 3}"></script>'
            .replace('SHA1', sha1))
        self.assertEqual(expected_output, unescape(result))

    def test_nested_blocks(self):
        # Nested JSX blocks are not allowed
        test_content = '''
        {% load jsx %}
        {% jsx %}
            <Component1>
                {% jsx %}<Component2/>{% endjsx %}
            </Component1>
        {% endjsx %}'''
        with self.assertRaises(TemplateSyntaxError) as raise_context:
            self.try_it(test_content, None, raw=True,)
            exc = raise_context.exc
            self.assertIn('jsx blocks cannot be nested in a template', str(exc))



from __future__ import print_function, unicode_literals

import os
import re
import hashlib

from django.apps import apps
import django.template
from django.template.backends.django import DjangoTemplates
from django.core.management.base import BaseCommand

# Regex matching all JSX blocks in a template
R_JSX = re.compile(r'\{% *jsx *%\}(.*?)\{% *endjsx *%\}', re.DOTALL)

# Regex to spot the beginning of an HTML element in JSX text
R_COMPONENT = re.compile(r'<(\w+)')

START_JS = """
import React from 'react';
import ReactDOM from 'react-dom';
var jsx_registry = {};
"""

# <script type="script/django-jsx" ...>

END_JS = """
function renderAllDjangoJSX(COMPONENTS) {
    Array.prototype.forEach.call(
        // Find all "django-jsx" scripts which are hooks to render and inject react components
        document.querySelectorAll('script[type^=script][type$=django-jsx]'),
        function(el) {

            // Extract serialized context data for rendering the component and get the component
            // from our database
            let ctx = JSON.parse(el.dataset.ctx)
            let component = jsx_registry[el.dataset.sha1](COMPONENTS, ctx)

            // Actually render and place the component into the pgae:
            // 1) Create a placeholder to render the component into
            // 2) Render the component into the placeholder
            // 3) Replace the placeholder with the actual component
            // 4) Remove the <script> hook to clean up

            el.insertAdjacentHTML("afterend", "<span></span>")

            ReactDOM.render(component, el.nextSibling)

            //el.parentNode.replaceChild(el.nextSibling.children[0], el.nextSibling)
            //el.parentNode.removeChild(el)
        }
    )
}

jsx_registry.renderAllDjangoJSX = renderAllDjangoJSX;
export default jsx_registry;
"""


class Command(BaseCommand):
    def add_arguments(self, parser):
        # Named (optional) arguments
        parser.add_argument(
            '-o'
            '--output',
            action='store',
            dest='output',
        )

    def handle(self, *args, **kwargs):
        output = None
        if kwargs['output']:
            output = open(kwargs['output'], 'w')

        compile_templates(list_template_files(), output)

        if output is not None:
            output.close()


def list_template_files():
    """
    Return list of template files everywhere Django looks for them.
    """
    engines = django.template.engines

    template_dirs = []
    # 'engines' is not a dictionary, it just behaves like one in some ways
    for engine_name in engines:
        engine = engines[engine_name]
        if isinstance(engine, DjangoTemplates):
            # We only handle Django templates
            template_dirs.extend(engine.template_dirs)

    template_list = []
    for each in template_dirs:
        for dir, dirnames, filenames in os.walk(each):
            for filename in filenames:
                template_list.append(os.path.join(dir, filename))

    for appname, app in apps.app_configs.items():
        app_dir = os.path.dirname(app.module.__file__)
        for dir, dirnames, filenames in os.walk(os.path.join(app_dir, "templates")):
            for filename in filenames:
                template_list.append(os.path.join(dir, filename))

    return template_list


def compile_templates(template_list, output=None):
    """
    Write a jsx_registry.js file to output (or stdout if output is None),
    containing boilerplate at top and bottom, and a jsx_registry entry for
    each jsx block found in any of the template files listed in `template_list`.
    :param template_list: A list of template filenames.
    :param output: A file-like object to write to, or None.
    :return: nothing
    """
    print(START_JS, file=output)
    for template in template_list:
        first = True
        try:
            content = open(template).read()
        except (IOError, UnicodeDecodeError):
            pass
        else:
            jsx_blocks = re.findall(R_JSX, content)
            for jsx in jsx_blocks:
                if first:
                    # Add comment indicating the template that these blocks came from.
                    # Can help with debugging.
                    first = False
                    print('/* %s */' % template, file=output)
                hash = hashlib.sha1(jsx.encode('utf-8')).hexdigest()

                jsx = jsx.strip()
                components = set(re.findall(R_COMPONENT, jsx))
                component_statements = []
                # Sort for repeatable output, making for easier debugging and testing
                for component in sorted(components):
                    component_statements.append(
                        "if (Object.hasOwnProperty.call(COMPONENTS, '%(component)s')) "
                        "var {%(component)s} = COMPONENTS;\n" % locals())
                component_statements.append('return (%(jsx)s);' % locals())
                component_statements = ''.join(component_statements)

                print('jsx_registry["%(hash)s"] = '
                      '(COMPONENTS, ctx) => {\n%(component_statements)s\n}' % locals(), file=output)
    print(END_JS, file=output)



import re
import json
from hashlib import sha1
import logging

from django import template
from django.template import TemplateSyntaxError
from django.utils.html import escape
from django.template.base import TOKEN_VAR, TOKEN_BLOCK, Variable, VariableDoesNotExist

# Regex to find references to context that start with "ctx."
# and look like "ctx.foo.bar" or "ctx.3.xyz" etc.
R_CTXEXPR = re.compile(r'\.*ctx\.([A-Za-z][\d\w\.]*)')

logger = logging.getLogger(__name__)
register = template.Library()


def set_nested(dictionary, key, value):
    """
    Key is a string with optional "." in it, e.g. "foo.bar".
    Will set value in the dictionary so that
    dictionary['foo']['bar'] = value
    """
    elts = key.split('.')
    if len(elts) == 1:
        # Simple key, just set the value in the dictionary ...
        if elts[0] not in dictionary:
            # ... but only if the key is not already there.
            dictionary[elts[0]] = value
    else:
        # Dotted key. Need to make sure the item with the first
        # part of the key is a dictionary, then set the value
        # into that dictionary using the key with the first part
        # stripped off.

        if not isinstance(dictionary.get(elts[0], None), dict):
            # Either top level expression is not yet in the dictionary, or it
            # is there, but it is not itself a dictionary. It must be a
            # dictionary so we can put our nested value inside of it, so set
            # it to a dictionary.
            dictionary[elts[0]] = {}
        new_key = '.'.join(elts[1:])
        set_nested(dictionary[elts[0]], new_key, value)


def serialize_opportunistically(context, expressions):
    """
    :param context: A template context
    :param expressions: A list of strings that refer to the context, e.g. "foo.bar" or "baz.1"
    :return: A string containing a JSON dump of a dictionary representing the parts of the
      context that are referred to in the expressions, resolved to their final values.
      In other words, a snapshot of the current context, limited to the listed names.
    """
    ctx = {}
    for expression in expressions:
        try:
            value = Variable(expression).resolve(context)
        except VariableDoesNotExist:
            logger.debug(
                "JSX block refers to ctx.%s, but there's no variable by that name "
                "in the Django template context.", expression)
            if context.template:
                string_if_invalid = context.template.engine.string_if_invalid
            else:
                string_if_invalid = ''
            if '%s' in string_if_invalid:
                value = string_if_invalid % expression
            else:
                value = string_if_invalid
        set_nested(ctx, expression, value)
    ctx = json.dumps(ctx)
    return ctx


@register.tag
def jsx(parser, token):
    """

    `jsx` is a block tag for Django templates.

    The block should contain JSX (magic HTML-ish markup for React).
    It can also contain the usual Django template stuff.  (But it
    can't contain another jsx tag.)

    In the JavaScript in the JSX, you can refer to values from
    the Django template context using the `ctx` variable and
    dot-notation. E.g. to access "field.name" from the template
    context, use "ctx.field.name" in the JavaScript.

    See also the compilejsx management command, which provides the
    JavaScript to use the data that this template tag puts into the
    HTML.

    When rendered, the block will turn into an empty script tag whose
    data attributes will contain:

    data-sha1) The sha1 hex digest of the body of the block. (Used by
    jsx_registry.js to find this script tag.)

    data-ctx) A serialized copy of the contents of the template context
    at the point where this block was, filtered to the bits that are referred
    to in the JSX.
    """

    text = []

    while parser.tokens:
        token = parser.next_token()

        if token.contents == 'endjsx':
            break

        if token.contents == 'jsx':
            raise TemplateSyntaxError("jsx blocks cannot be nested in a template")

        if token.token_type == TOKEN_VAR:
            text.append('{{')
        elif token.token_type == TOKEN_BLOCK:
            text.append('{%')

        text.append(token.contents)

        if token.token_type == TOKEN_VAR:
            text.append('}}')
        elif token.token_type == TOKEN_BLOCK:
            text.append('%}')

    return JsxNode(''.join(text))


class JsxNode(template.Node):
    def __init__(self, jsx):
        self.jsx = jsx

    def render(self, context):
        expressions = re.findall(R_CTXEXPR, self.jsx)
        ctx = serialize_opportunistically(context, expressions)
        return '<script type="script/django-jsx" data-sha1="%s" data-ctx="%s"></script>' % \
               (sha1(self.jsx.encode('utf-8')).hexdigest(), escape(ctx))



from setuptools import setup, find_packages

setup(
    name='django-jsx',
    version='0.5.0',
    author='Calvin Spealman',
    author_email='calvin@caktusgroup.com',
    packages=find_packages(exclude=['sample_project']),
    include_package_data=True,
    license='BSD',
    description='Integration library for React/JSX and Django',
    classifiers=[
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
    ],
)
